float4x4 worldViewProjI; float postfx;
// Cubes and Spheres
// by @paulofalcao


uniform float2 resolution = float2(1,1);
uniform float time;
uniform float2 mouse;

//Util Start
float PI=3.14159265;

float2 ObjUnion(
  in float2 obj0,
  in float2 obj1)
{
  if (obj0.x<obj1.x)
    return obj0;
  else
    return obj1;
}

float2 sim2d(
  in float2 p,
  in float s)
{
   float2 ret=p;
   ret=p+s/2.0;
   ret=frac(ret/s)*s-s/2.0;
   return ret;
}

float3 stepspace(
  in float3 p,
  in float s)
{
  return p-fmod(p-s/2.0,s);
}

float3 phong(
  in float3 pt,
  in float3 prp,
  in float3 normal,
  in float3 light,
  in float3 color,
  in float spec,
  in float3 ambLight)
{
   float3 lightv=normalize(light-pt);
   float diffuse=dot(normal,lightv);
   float3 refl=-reflect(lightv,normal);
   float3 viewv=normalize(prp-pt);
   float specular=pow(max(dot(refl,viewv),0.0),spec);
   return (max(diffuse,0.0)+ambLight)*color+specular;
}

//Util End

//Scene Start

float2 obj(in float3 p)
{ 
  float3 fp=stepspace(p,2.0);;
  float d=sin(fp.x*0.3+time*4.0)+cos(fp.z*0.3+time*2.0);
  p.y=p.y+d;
  p.xz=sim2d(p.xz,2.0);
  float c1=length(max(abs(p)-float3(0.6,0.6,0.6),0.0))-0.35;
  float c2=length(p)-1.0;
  float cf=sin(time)*0.5+0.5;
  return float2(lerp(c1,c2,cf),1.0);
}

float3 obj_c(float3 p){
  float2 fp=sim2d(p.xz-1.0,4.0);
  if (fp.y>0.0) fp.x=-fp.x;
  if (fp.x>0.0) return float3(0.2,0.5,1.0);
    else return float3(1.0,0.5,0.2);   
}

//Scene End

float raymarching(
  in float3 prp,
  in float3 scp,
  in int maxite,
  in float precis,
  in float startf,
  in float maxd,
  out float objid)
{ 
  const float3 e=float3(0.1,0,0.0);
  float2 s=float2(startf,0.0);
  float3 c,p,n;
  float f=startf;
  for(int i=0;i<256;i++){
    if (abs(s.x)<precis||f>maxd||i>maxite) break;
    f+=s.x;
    p=prp+scp*f;
    s=obj(p);
    objid=s.y;
  }
  if (f>maxd) objid=-1.0;
  return f;
}

float3 camera(
  in float3 prp,
  in float3 vrp,
  in float3 vuv,
  in float vpd,
  in float2 gl_FragCoord)
{
  float2 vPos=-1.0+2.0*gl_FragCoord.xy/resolution.xy;
  float3 vpn=normalize(vrp-prp);
  float3 u=normalize(cross(vuv,vpn));
  float3 v=cross(vpn,u);
  float3 scrCoord=prp+vpn*vpd+vPos.x*u*resolution.x/resolution.y+vPos.y*v;
  return normalize(scrCoord-prp);
}

float3 normal(in float3 p)
{
  //tetrahedron normal
  const float n_er=0.01;
  float v1=obj(float3(p.x+n_er,p.y-n_er,p.z-n_er)).x;
  float v2=obj(float3(p.x-n_er,p.y-n_er,p.z+n_er)).x;
  float v3=obj(float3(p.x-n_er,p.y+n_er,p.z-n_er)).x;
  float v4=obj(float3(p.x+n_er,p.y+n_er,p.z+n_er)).x;
  return normalize(float3(v4+v1-v3-v2,v3+v4-v1-v2,v2+v4-v3-v1));
}

float3 render(
  in float3 prp,
  in float3 scp,
  in int maxite,
  in float precis,
  in float startf,
  in float maxd,
  in float3 background,
  in float3 light,
  in float spec,
  in float3 ambLight,
  out float3 n,
  out float3 p,
  out float f,
  out float objid)
{ 
  objid=-1.0;
  f=raymarching(prp,scp,maxite,precis,startf,maxd,objid);
  if (objid>-0.5){
    p=prp+scp*f;
    float3 c=obj_c(p);
    n=normal(p);
    float3 cf=phong(p,prp,n,light,c,spec,ambLight);
    return float3(cf);
  }
  f=maxd;
  return float3(background); //background color
}

float4 main(float2 gl_FragCoord: TEXCOORD0, float4 gl_FragColor: COLOR ) : COLOR{
 
  //Camera animation
  float3 vuv=float3(0,1,0);
  float3 vrp=float3(time*4.0,0.0,0.0);
  float mx=mouse.x*PI*2.0;
  float my=mouse.y*PI/2.01; 
  float3 prp=vrp+float3(cos(my)*cos(mx),sin(my),cos(my)*sin(mx))*12.0; //Trackball style camera pos
  float vpd=1.5;
  float3 light=prp+float3(5.0,0,5.0);
  
  float3 scp=camera(prp,vrp,vuv,vpd,gl_FragCoord);
  float3 n,p;
  float f,o;
  const float maxe=0.01;
  const float startf=0.1;
  const float3 backc=float3(0.0,0.0,0.0);
  const float spec=8.0;
  const float3 ambi=float3(0.1,0.1,0.1);
  
  float3 c1=render(prp,scp,256,maxe,startf,60.0,backc,light,spec,ambi,n,p,f,o);
  c1=c1*max(1.0-f*.015,0.0);
  float3 c2=backc;
  if (o>0.5){
    scp=reflect(scp,n);
    c2=render(p+scp*0.05,scp,32,maxe,startf,10.0,backc,light,spec,ambi,n,p,f,o);
  }
  c2=c2*max(1.0-f*.1,0.0);
  gl_FragColor=float4(c1.xyz*0.75+c2.xyz*0.25,1.0);
            return gl_FragColor;

  
}
// Vertex Shader
struct VS_INPUT
{
    float3 position	: POSITION;
    float2 texture0     : TEXCOORD0;
};

struct VS_OUTPUT
{
     float4 hposition : POSITION;
     float2 texture0  : TEXCOORD0;
};

VS_OUTPUT myvs( VS_INPUT IN )
{
    VS_OUTPUT OUT;
    if (postfx == 0) {
      OUT.hposition = mul( worldViewProjI, float4(IN.position.x ,IN.position.y ,IN.position.z, 1) ); 
    } else {
      OUT.hposition = float4(IN.position.x ,IN.position.y ,IN.position.z, 1);
    }    
    OUT.texture0 = IN.texture0;
    return OUT;
}
// ---------------------------------------------
technique Start
{
    pass p1  
    {
        VertexShader = compile vs_3_0 myvs();
        PixelShader = compile ps_3_0 main();
    }
}