// volume explosion shader
// simon green / nvidia 2012
// http://developer.download.nvidia.com/assets/gamedev/files/gdc12/GDC2012_Mastering_DirectX11_with_Unity.pdf
// @mod* rotwang

float4x4 worldViewProjI;
uniform float2 resolution = float2(640,480) ; // Screensize;
uniform float time;
uniform float2 mouse;



// parameters to play with
int _MaxSteps = 32;
float _StepDistanceScale = 0.72;
float _MinStep = 0.01;
float _DistThreshold = 0.05;

int _VolumeSteps = 32;
float _StepSize = 0.01;
float _Density = 0.1;

float _SphereRadius = 0.95;
float _NoiseFreq = 8.0;
float _NoiseAmp = -0.3;
float3 _NoiseAnim = float3(0.93, 0, 0);




// iq's nice integer-less noise function
// matrix to rotate the noise octaves
// Um die folgenden remarkten Zeilen geht es, ich bekomms nicht lauffhig hin.
// Das float m = 3.5; ist von mir und kannst du lschen wenn du das mit dem mat3 m hinbekommen solltest ?!

float3x3 m = float3x3( 0.00, 0.80, 0.60,
-0.80, 0.36, -0.48,
-0.60, -0.48, 0.64 );
//float m = 3.5;



float hash( float n )
{ return frac(sin(n)*43758.5453); }



float noise( in float3 x )
{ float3 p = floor(x);
float3 f = frac(x);
f = f*f*(3.0-2.0*f);
float n = p.x + p.y*57.0 + 113.0*p.z;
float res = lerp(lerp(lerp( hash(n+ 0.0), hash(n+ 1.0),f.x),
lerp( hash(n+ 57.0), hash(n+ 58.0),f.x),f.y),
lerp(lerp( hash(n+113.0), hash(n+114.0),f.x),
lerp( hash(n+170.0), hash(n+171.0),f.x),f.y),f.z);
return res; }


// kannst auch mal testeweise hier m[1] oder m[2] verwenden 

float fbm( float3 p )
{ float f;
f = 0.5000*noise( p );
p = m[0]*p*2.02;
f += 0.2500*noise( p );
p = m[0]*p*2.03;
f += 0.1250*noise( p );
p = m[0]*p*2.01;
f += 0.0625*noise( p );
p = m[0]*p*2.02;

f += 0.03125*abs(noise( p )); 
return f/0.9375; }


// distance field stuff
float sphereDist(float3 p, float4 sphere)
{ return length(p - sphere.xyz) - sphere.w; }


// returns signed distance to nearest surface
// displace is displacement from original surface (0, 1)
float distanceFunc(float3 p, out float displace)
{ 
float d = length(p) - _SphereRadius; // distance to sphere
//float d = length(p) - (sin(time*0.25)+0.8); // animated radius

// offset distance with pyroclastic noise
//p = normalize(p) * _SphereRadius; // project noise points to sphere surface
displace = fbm(p*_NoiseFreq + _NoiseAnim*time);
d += displace * _NoiseAmp;

return d;
}


// calculate normal from distance field
float3 dfNormal(float3 pos)
{
float eps = 0.001;
float3 n;
float s;
#if 0
// central difference
n.x = distanceFunc( float3(pos.x+eps, pos.y, pos.z), s ) - distanceFunc( float3(pos.x-eps, pos.y, pos.z), s );
n.y = distanceFunc( float3(pos.x, pos.y+eps, pos.z), s ) - distanceFunc( float3(pos.x, pos.y-eps, pos.z), s );
n.z = distanceFunc( float3(pos.x, pos.y, pos.z+eps), s ) - distanceFunc( float3(pos.x, pos.y, pos.z-eps), s );
#else
// forward difference (faster)
float d = distanceFunc(pos, s);
n.x = distanceFunc( float3(pos.x+eps, pos.y, pos.z), s ) - d;
n.y = distanceFunc( float3(pos.x, pos.y+eps, pos.z), s ) - d;
n.z = distanceFunc( float3(pos.x, pos.y, pos.z+eps), s ) - d;
#endif

return normalize(n);
}

// color gradient
// this should be in a 1D texture really
float4 gradient(float x)
{
float4 c0 = float4(3, 3, 1, 1); // hot white
float4 c1 = float4(1, 0.5, 0, 1); // yellow
float4 c2 = float4(1, 0, 0, 1); // red
float4 c3 = float4(0.2, 0.2, 0.2, 1); // grey

float t = frac(x*3.0);
float4 c;
if (x < 0.333333) {
c = lerp(c0, c1, t);
} else if (x < 0.666666) {
c = lerp(c1, c2, t);
} else {
c = lerp(c2, c3, t);
}

return c;
}


// shade a points based on position and displacement from surface
float4 shade(float3 p, float displace)
{ 
// lookup in color gradient
displace = displace*2.7 - 0.5;
displace = clamp(displace, 0.0, 0.99);
float4 c = gradient(displace);


// lighting
float3 n = dfNormal(p);
float diffuse = n.z*0.35+0.85;
//float diffuse = max(0.0, n.z);
c.rgb = lerp(c.rgb, c.rgb*diffuse, clamp((displace-0.5)*2.0, 0.0, 1.0));

//return float4(float3(displace), 1);
//return float4(dfNormal(p)*float3(0.5)+float3(0.5), 1);
//return float4(diffuse);
//return gradient(displace);
return c;
}

// procedural volume
// maps position to color
float4 volumeFunc(float3 p)
{
float displace;
float d = distanceFunc(p, displace);
float4 c = shade(p, displace);
return c;
}

// sphere trace
// returns hit position
float3 sphereTrace(float3 rayOrigin, float3 rayDir, out bool hit, out float displace)
{
float3 pos = rayOrigin;
hit = false;
displace = 0.0; 
float d;
//float3 hitPos;
float disp;
for(int i=0; i<_MaxSteps; i++) {
d = distanceFunc(pos, disp);
if (d < _DistThreshold) {
hit = true;
displace = disp;
//hitPos = pos;
//break; // early exit from loop doesn't work in ES?
}
//d = max(d, _MinStep);
pos += rayDir*d*_StepDistanceScale;
}

return pos;
//return hitPos;
}


// ray march volume from front to back
// returns color
float4 rayMarch(float3 rayOrigin, float3 rayStep, out float3 pos)
{
float4 sum = float4(0, 0, 0, 0);
pos = rayOrigin;
for(int i=0; i<_VolumeSteps; i++) {
float4 col = volumeFunc(pos);
col.a *= _Density;
col.a = min(col.a, 1.0);

// pre-multiply alpha
col.rgb *= col.a;
sum = sum + col*(1.0 - sum.a); 
#if 0
// exit early if opaque
if (sum.a > _OpacityThreshold)
break;
#endif 
pos += rayStep;
}
return sum;
}

float4 main(float2 gl_FragCoord: TEXCOORD0, float4 gl_FragColor: COLOR ) : COLOR
{ gl_FragCoord.y = 1 - gl_FragCoord.y; gl_FragCoord = gl_FragCoord * resolution;

float t = time*0.125;
float ust = sin(t)*0.5+0.5;
float2 q = gl_FragCoord.xy / resolution.xy;
float2 p = -1.0 + 2.0 * q;
p.x *= resolution.x/resolution.y;

float rotx = 0.0; // mouse.y*4.0;
float roty = 0.0; // mouse.x*4.0;

// camera
float3 ro = ust*4.0*normalize(float3(cos(roty), cos(rotx), sin(roty)));
float3 ww = normalize(float3(0.0,0.0,0.0) - ro);
float3 uu = normalize(cross( float3(0.0,1.0,0.0), ww ));
float3 vv = normalize(cross(ww,uu));
float3 rd = normalize( p.x*uu + p.y*vv + 1.5*ww );

// sphere trace distance field
bool hit;
float displace;
float3 hitPos = sphereTrace(ro, rd, hit, displace);

float4 col = float4(0, 0, 0, 1);
if (hit) {
// shade
col = shade(hitPos, displace); // opaque version
//col = rayMarch(hitPos, rd*_StepSize, hitPos); // volume render
}
return col; }


// Vertex Shader
struct VS_INPUT
{
float3 position : POSITION;
float2 texture0 : TEXCOORD0;
};

struct VS_OUTPUT
{
float4 hposition : POSITION;
float2 texture0 : TEXCOORD0;
};

VS_OUTPUT myvs( VS_INPUT IN )
{ VS_OUTPUT OUT;
OUT.hposition = float4(IN.position.x ,IN.position.y ,IN.position.z, 1);
OUT.texture0 = IN.texture0;
return OUT; }

// ---------------------------------------------
technique Start
{
pass p1 
{
VertexShader = compile vs_3_0 myvs();
PixelShader = compile ps_3_0 main();
}
}

